@@ -10,13 +10,11 @@ if t.TYPE_CHECKING:
 
 
 @t.overload
-def normalize_identifiers(expression: E, dialect: DialectType = None) -> E:
-    ...
+def normalize_identifiers(expression: E, dialect: DialectType = None) -> E: ...
 
 
 @t.overload
-def normalize_identifiers(expression: str, dialect: DialectType = None) -> exp.Identifier:
-    ...
+def normalize_identifiers(expression: str, dialect: DialectType = None) -> exp.Identifier: ...
 
 
 def normalize_identifiers(expression, dialect=None):
@@ -55,10 +53,8 @@ def normalize_identifiers(expression, dialect=None):
     if isinstance(expression, str):
         expression = exp.parse_identifier(expression, dialect=dialect)
 
-    def _normalize(node: E) -> E:
+    for node in expression.walk(prune=lambda n: n.meta.get("case_sensitive")):
         if not node.meta.get("case_sensitive"):
-            exp.replace_children(node, _normalize)
-            node = dialect.normalize_identifier(node)
-        return node
+            dialect.normalize_identifier(node)
 
-    return _normalize(expression)
+    return expression
